/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Sonatype, Nexus*/

/**
 * Staging repositories master grid.
 *
 * @since 2.3
 */
NX.define('Nexus.staging.repository.RepositoriesGrid', {
    extend: 'Ext.grid.GridPanel',

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.prefs.DateFormat',
        'Nexus.staging.repository.RepositoriesMediator',
        'Nexus.ext.GridFilterBox'
    ],

    /**
     * @override
     */
    initComponent: function () {
        var self = this,
            mediator = Nexus.staging.repository.RepositoriesMediator,
            icons = Nexus.staging.Icons;

        self.ds = mediator.repositoryStore;
        mediator.repositoryStore.on('beforeload', self.rememberSelection, self);
        mediator.repositoryStore.on('load', self.recallSelection, self);

        self.loadMask = {
            msg: 'Loading...',
            msgCls: 'loading-indicator'
        };

        self.viewConfig = {
          emptyText: 'No staging repositories available',
          emptyTextWhileFiltering: 'No staging repositories matched criteria: {criteria}'
        };

        self.stripeRows = true;

        self.sm = NX.create('Ext.grid.CheckboxSelectionModel', {
            listeners: {
                selectionchange: {
                    fn: self.selectionChanged,
                    scope: self
                }
            }
        });

        var columns = [
            // checkbox
            self.sm,

            // icon
            {
                width: 30, // some padding here for badges
                resizable: false,
                sortable: false,
                fixed: true,
                hideable: false,
                menuDisabled: true,
                renderer: function (value, metaData, record) {
                    return icons.renderRepositoryWithBadge(record.data);
                }
            },
            {
                id: 'repositoryId',
                width: 250,
                header: 'Repository',
                dataIndex: 'repositoryId',
                sortable: true
            },
            {
                id: 'profileName',
                width: 100,
                header: 'Profile',
                dataIndex: 'profileName',
                sortable: true
            },
            {
                id: 'type',
                width: 60,
                header: 'Status',
                dataIndex: 'type',
                sortable: true
            },
            {
                id: 'releaseRepositoryId',
                width: 110,
                header: 'Release To',
                dataIndex: 'releaseRepositoryId',
                sortable: true,
                hidden: true
            },
            {
                id: 'parentGroupId',
                width: 100,
                header: 'Promoted To',
                dataIndex: 'parentGroupId',
                sortable: true,
                hidden: true
            },
            {
                id: 'userId',
                header: 'Owner', // Using owner here for clarity
                dataIndex: 'userId',
                sortable: true,
                hidden: true
            },
            {
                id: 'created',
                width: 150,
                header: 'Created',
                dataIndex: 'created',
                sortable: true,
                xtype: 'datecolumn',
                format: Nexus.prefs.DateFormat.forName('datetime')['short'],
                hidden: true
            },
            {
                id: 'updated',
                width: 150,
                header: 'Updated',
                dataIndex: 'updated',
                sortable: true,
                xtype: 'datecolumn',
                format: Nexus.prefs.DateFormat.forName('datetime')['short']
            },
            {
                id: 'userAgent',
                width: 100,
                header: 'User Agent',
                dataIndex: 'userAgent',
                sortable: true,
                hidden: true
            },

            {
                id: 'description',
                width: 100,
                header: 'Description',
                dataIndex: 'description',
                sortable: true
            }
        ];
        // Allow contributors to add to the view.
        Nexus.staging.repository.RepositoriesMediator.fireEvent('staging.repositoriesGrid.configureColumns', columns);

        self.cm = NX.create('Ext.grid.ColumnModel', {
            columns: columns
        });
        self.autoExpandColumn = 'description';

        // FIXME: Toolbar, and actions for them should really be handled by another class
        // FIXME: Should try to avoid using component ids here, a field ref would be preferable, or itemId instead.

        self.tbar = [
            {
                text: 'Refresh',
                tooltip: 'Refresh staging repositories',
                iconCls: icons.get('refresh').cls,
                handler: function () {
                    mediator.refreshHandler();
                }
                // always enabled
            },
            {
                text: 'Close',
                itemId: 'close',
                iconCls: icons.get('closeRepository').cls,
                tooltip: 'Close staging repositories',
                handler: function () {
                    mediator.closeHandler(self.getSelectionModel().getSelections());
                },
                disabled: true
            },
            {
                text: 'Promote',
                itemId: 'promote',
                iconCls: icons.get('promoteRepository').cls,
                tooltip: 'Promote staging repositories',
                handler: function () {
                    mediator.promoteHandler(self.getSelectionModel().getSelections());
                },
                disabled: true
            },
            {
                text: 'Release',
                itemId: 'release',
                iconCls: icons.get('releaseRepository').cls,
                tooltip: 'Release staging repositories',
                handler: function () {
                    mediator.releaseHandler(self.getSelectionModel().getSelections());
                },
                disabled: true
            },
            {
                text: 'Drop',
                itemId: 'drop',
                iconCls: icons.get('dropRepository').cls,
                tooltip: 'Drop staging repositories',
                handler: function () {
                    mediator.dropHandler(self.getSelectionModel().getSelections());
                },
                disabled: true
            },

            '->',
            NX.create('Nexus.ext.GridFilterBox', {
              filteredGrid: self
            })
        ];

        self.constructor.superclass.initComponent.apply(self, arguments);
    },

    /**
     * @private
     */
    selectionChanged: function (sm) {
        var self = this,
            records,
            provider,
            i,
            mediator = Nexus.staging.repository.RepositoriesMediator,
            sp = Sonatype.lib.Permissions,
            closable,
            promotable,
            relesable,
            droppable;

        function disableAll() {
            closable = false;
            promotable = false;
            relesable = false;
            droppable = false;
        }

        // start out disabled
        disableAll();

        // if there is a selection then we have to work a little to figure out what actions are available
        if (sm.getCount() !== 0) {
            // we have a selection, enable all actions which the user has permission to use
            closable = sp.checkPermission('nexus:stagingfinish', sp.CREATE);
            promotable = sp.checkPermission('nexus:stagingpromote', sp.CREATE);
            relesable = sp.checkPermission('nexus:stagingpromote', sp.CREATE);
            droppable = sp.checkPermission('nexus:stagingdrop', sp.CREATE);

            records = sm.getSelections();
            provider = null;

            // scan through all selections and flip off any actions which are not valid for the selection
            for (i = 0; i < records.length; i = i + 1) {
                // if any record is transitioning disable all toolbar actions and top
                if (records[i].get('transitioning') === true) {
                    disableAll();
                    break;
                }

                // NOTE: Not really sure why this check is done ATM, since IIUC there is only 1 provider 'maven2'

                // the first items provider is used to compare with all other selected repositories
                if (provider === null) {
                    provider = records[i].get('provider');
                }
                // if there is a mismatch, then disable all actions
                else if (provider !== records[i].get('provider')) {
                    disableAll();
                    break;
                }

                if (records[i].get('type') === 'open') {
                    relesable = false;
                    promotable = false;
                }
                else if (records[i].get('type') === 'closed') {
                    closable = false;
                }
                else if (records[i].get('type') === 'promoted') {
                    disableAll();
                    break;
                }
                else if (records[i].get('type') === 'released') {
                    closable = false;
                    relesable = false;
                    promotable = false;
                    break;
                }
            }

            // if there are no promotion profiles, then disable promote action
            if (!mediator.profileStore.hasGroups) {
                promotable = false;
            }
        }

        // toggle enabled state of buttons by itemId
        function toggleEnabled(itemId, enable) {
            var button = self.getTopToolbar().getComponent(itemId);
            NX.assert(button);

            //self.logDebug('Toggling toolbar action enabled:', itemId, enable);

            if (enable) {
                button.enable();
            }
            else {
                button.disable();
            }
        }

        // toggle toolbar buttons
        toggleEnabled('close', closable);
        toggleEnabled('promote', promotable);
        toggleEnabled('release', relesable);
        toggleEnabled('drop', droppable);
    },

    /**
     * @private
     */
    selectedRecords: undefined,

    /**
     * Remember what records are selected.
     *
     * @private
     */
    rememberSelection: function () {
        this.selectedRecords = this.getSelectionModel().getSelections();
    },

    /**
     * Recall selection from previous remembered selection.
     *
     * @private
     */
    recallSelection: function () {
        var self = this,
            toSelect = [];

        if (self.selectedRecords === undefined || self.selectedRecords.length === 0) {
            return;
        }

        Ext.each(self.selectedRecords, function (record) {
            record = self.getStore().getById(record.id);
            if (!Ext.isEmpty(record)) {
                toSelect.push(record);
            }
        });

        self.getSelectionModel().selectRecords(toSelect);
    }
});
